# RouterSDK

## Factory

Create a new RouterSDK instance.

```js
import { RouterFactorySDK } from "@allianceblock/abridge-sdk";

const routerFactory = new RouterFactorySDK();
const sdk = await routerFactory.create(
  chainType, // 'EVM' or 'SOLANA'
  signerOrProvider, // JsonRpcSigner (userWallet) | JsonRpcProvider
  optionalParams
);
```

RouterFactorySDK returns an instance of SolanaRouter or EvmRouter.
Each of these instances inherits from the RouterSDK class.

Methods that are common to both networks are created in RouterSDK and used equally.
The methods that require a specific development due to the type of network must be created in their own class.

In the same way common or specific props can be set.

## optionalParams

Common params

- @property nftValidators: string
- @property abridgeBackend: string
- @property frontEndVersion: number

EVM params

- @property validators: object
- @property legacyRouterABI: legacyErc20/IRouterDiamond
- @property legacyRouterAddress: address
- @property legacyRouterProxyABI: legacyErc20/RouterProxy
- @property legacyRouterProxyAddress: address
- @property wrappedErc721ABI: WrappedERC721
- @property teleportABI: ITeleport
- @property nftRouterABI: NFTIRouterDiamond
- @property erc20RouterABI: ERC20IRouterDiamond
- @property erc20RouterAddress: address
- @property erc20PermitABI: ERC20Permit
- @property threshold: number
- @property validatorTargetVersion: string
- @property nftValidatorTargetVersion: string

Solana params

- @property teleportIDL: object
- @property ERC20RouterIDL: object

## methods

### Legacy

The legacy methods correspond to version 2 of the router, these methods are kept for compatibility with previous versions of the SDK.

### Router methods

`setParams(optionalParams)`

Checks if some of the optional parameters are given and sets them, if there is a missing optional parameter a default value is used

`estimateErc20AutoClaim(sourceChainId, targetChainId, tokenAddress, amount, senderAddress, receiverAddress)`

returns the estimated gas for the autoClaim method

```js
{ fee: <ethers.BigNumber>, expiry: <number>, signature: <hex string> }
```

```js
sourceChainId; // number
targetChainId; // number
tokenAddress; // address, like 0x...
amount; // string, like 0.1
senderAddress; // address, like 0x...
receiverAddress; // address, like 0x...
```

`async getClaimsLegacy(userAddress, targetChainId, page)`

Returns all the transactions for `userAddress` held in the specified chain
If `page` is not specified, all claims are returned.
The header returns two additional params:

```
x-total-count: Number of total results
x-total-pages: Number of all pages
```

```js
userAddress; // string - User wallet address
targetChainId; // string - ID of the target chain
page; // [optional] Get the claims per page and return the requested page
```

`getErc20Claims(targetChainId, userAddress)`

`getRandomProperty(obj)`

This method receives an object and returns a random property from it

`async getValidatorVersionLegacy()`

Returns the version of the current validator that is attached to.

`async getWrappedTokensLegacy(chainId)`

Returns all the wrapped tokens from the specified chainId

```js
chainId; // string - ID of the chain
```

`loadTokenMetadata(metadataUrl)`
This method receives a metadata url and returns the metadata of the token in the following format:

```js
{
  name: string;
  symbol: string;
  decimals: number;
  logoURI: string;
  description: string;
  external_url: string;
  attributes: Array<{
    trait_type: string;
    value: string;
  }>;
}
```

`setValidator(url)`

Set validatorUrl

### Networks methods

`async allowance(tokenAddress, userAddress)`

Returns the maximum approved amount of a token
Returns ({ balance, tokenDecimals })

```js
tokenAddress; // string - Address of the deployed token
userAddress; // string - User wallet address
```

`async approve(tokenAddress, amount, bridgedTokenIsNativeTokenOrNonDefaultFee)`

Approve `amount` of token
Returns unsigned transaction

```js
tokenAddress; // string - Address of the deployed token
amount; // number - Amount of token to be approved
bridgedTokenIsNativeTokenOrNonDefaultFee; // Boolean
```

`async approveMaxInt(tokenAddress, bridgedTokenIsNativeTokenOrNonDefaultFee)`

Calls _approve_ with _ethers.constants.MaxUint256_ amount
Returns unsigned transaction

```js
tokenAddress; // string - Address of the deployed token
bridgedTokenIsNativeTokenOrNonDefaultFee; // Boolean
```

`async balanceOf(tokenAddress, userAddress)`

Returns the amount of token with this _tokenAddress_ of the user ({ balance, tokenDecimals })

```js
tokenAddress; // string - Address of the deployed token
userAddress; // string - User wallet address
```

`async balanceOfMulticall(tokenAddresses, userAddress)`

Returns the number of tokens with this _tokenAddress_ of the user for each item in the array.

```js
tokenAddresses; // []string - Array of addresses of the deployed token
userAddress; // string - User wallet address
```

`async claim(transmission)`

Mints wrapped token or unlocks the native one
Returns unsigned claim transaction

```js
// transmission: info needed to claim the transaction
const {
  transmissionId,
  sourceChainId,
  transmissionSender,
  transmissionReceiver,
  dAppId,
  payload,
  signatures,
} = transmission;
```

`fixSignatures(signatures)`

Formats the given signatures and returns the fixed ones

```js
signatures; // signature[] - Signatures list of the validators
```

`async getALBTToken()`

Returns the _address, name, symbol and decimals_ of the ALBT token used for token FEE into the different networks

`async getExternalFeeLegacy()`

Returns the amount of external fee in WEI

`async getNativeTokenByWrappedAddress(wrappedTokenAddress)`

Returns the native token

```js
wrappedTokenAddress; // string - Address of the deployed wrapped token
```

`async getProxyFeeLegacy(tokenAddress)`

Returns the fee amount for the chosen token

```js
tokenAddress; // string - Address of the token
```

`async getServiceFee()`

Returns the amount of service fee in WEI

`async getTokenDecimals(tokenAddress)`

Returns the token decimals by given token address

```js
tokenAddress; // string - Address of the deployed token
```

`async getTokenDetails(tokenAddress, useAddress)`

Returns the token name, symbol and decimals by given token address
If the parameter userAddress is supplied, the balance is returned.

```js
tokenAddress; // string - Address of the deployed token
useAddress; // string - User wallet address
```

`async getTokenDetailsMulticall(tokenAddresses, useAddress)`

Returns the token name, symbol and decimals by given token address for each item in the array.
If the parameter userAddress is supplied, the balance is returned.

```js
tokenAddresses; // []string - Array of address of the deployed token
useAddress; // string - User wallet address
```

`async getTokenName(tokenAddress)`

Returns the token name by given token address

```js
tokenAddress; // string - Address of the deployed token
```

`async getTokenSymbol(tokenAddress)`

Returns the token symbol by given token address

```js
tokenAddress; // string - Address of the deployed token
```

`async getWrappedTokenDetails(nativeTokenAddress, nativeChainId)`

Returns the _address, name, symbol and decimals_ of the wrapped token

```js
nativeTokenAddress; // string - Address of the deployed native token
nativeChainId; // string - ID of the source chain
```

`async isClaimedLegacy(transfer)`

Returns the `claimed` status of the transfer

```js
transfer; // single transfer object
```

`async isNftsApproved(userAddress, collectionAddress)`
Checks if the token has been approved
Returns boolean

```js
userAddress; // string - User wallet address
collectionAddress; // string - Address of the deployed token
```

`async isTokenApproved(tokenAddress, userAddress, threshold)`

Checks if the token has been approved
Returns boolean

```js
tokenAddress; // string - Address of the deployed token
userAddress; // string - User wallet address
threshold; // (optional) number
```

`sendErc20(estimateOnly, targetChainId, tokenAddress, amount, receiverAddress feeTokenAddress, deliveryFeeEstimate, deliveryFeeExpiry, deliveryFeeSignature, v, r, s, deadline)`

Send transaction.

```js
/* control */
estimateOnly; // boolean - true if we need to call estimate gas only
/* egress mandatory-ish */
targetChainId; // number
tokenAddress; // address - could be AddressZero when bridging native chain currency
amount; // ethers.BigNumber
receiverAddress; // address
feeTokenAddress; // address - could be AddressZero
/* optional - auto-delivery */
deliveryFeeEstimate; // ethers.BigNumber
deliveryFeeExpiry; // number
deliveryFeeSignature; // hex string
/* optional - permit */
v;
r;
s;
deadline; // all should be 'undefined' if not using egressWithPermit
```

`async sendLegacy(targetChainId, tokenAddress, amount, receiver, feeTokenAddress, bridgedTokenIsNativeToken)`

Send transaction. The nativeToken is retrieved and based on the chainId of the nativeToken, one of the following methods in the Router contract are called:

```
- lock
- burnAndTransfer
- burn
```

Please Note: The `feeTokenAddress` needs to be passed if not native fee (ALBT/WALBT) is chosen.
Please Note: If we want to bridge nativeCurrency, we need to pass `bridgedTokenIsNativeToken`
Returns unsigned lock/burnAndTransfer/burn transaction which should be signed.

```js
targetChainId; // string - ID of the target chain
tokenAddress; // string - Address of the deployed token
amount; // number - Amount of token to be send
receiver; // Recipient wallet address
feeTokenAddress; // Address of the fee token
bridgedTokenIsNativeToken; // Boolean (in case that we are bridging native currency)
```

`async sendWithPermitLegacy(targetChainId, tokenAddress, amount, receiver, v, r, s, deadline, feeTokenAddress, bridgedTokenIsNativeToken)`

Send transaction with `Permit` enabled. The nativeToken is retrieved and based on the chainId of the nativeToken, one of the following methods in the Router contract are called:
Returns unsigned lockWithPermit/burnWithPermit transaction
Please Note: The `feeTokenAddress` needs to be passed if not native fee (ALBT/WALBT) is chosen.
Please Note: If we want to bridge nativeCurrency, we need to pass `bridgedTokenIsNativeToken`

```js
targetChainId; // string - ID of the target chain
tokenAddress; // string - Address of the deployed token
amount; // number - Amount of token to be send
receiver; // Recipient wallet address
v; // number
r; // any
s; // string
deadline; // number - Milliseconds
feeTokenAddress; // Address of the fee token
bridgedTokenIsNativeToken; // Boolean (in case that we are bridging native currency)
```

`v, r, s` represent one signature with different formatting
